@@ -1,21 +1,7 @@
 /*
- *   Copyright 2017, 2018, 2019 by Ivan Cukic <ivan.cukic (at) kde.org>
+ *   SPDX-FileCopyrightText: 2017, 2018, 2019 Ivan Cukic <ivan.cukic (at) kde.org>
  *
- *   This program is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU General Public License as
- *   published by the Free Software Foundation; either version 2 of
- *   the License or (at your option) version 3 or any later version
- *   accepted by the membership of KDE e.V. (or its successor approved
- *   by the membership of KDE e.V.), which shall act as a proxy
- *   defined in Section 14 of version 3 of the license.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "vault.h"
@@ -23,6 +9,7 @@
 #include <QDir>
 #include <QFutureWatcher>
 #include <QDBusMetaType>
+#include <QMessageBox>
 #include <QUrl>
 #include <QPointer>
 
@@ -85,7 +72,7 @@ public:
     enum DeletionState {
         Normal,
         DeletionBlocked,
-        DeletionScheduled
+        DeletionScheduled,
     };
     DeletionState deletionState = Normal;
 
@@ -192,19 +179,19 @@ public:
 
                 emit q->statusChanged(data->status);
 
-                if (newStatus == VaultInfo::Closed
+                if (newStatus == VaultInfo::Closed //
                         || newStatus == VaultInfo::Opened) {
                     emit q->isOpenedChanged(newStatus == VaultInfo::Opened);
                 }
 
-                if (oldStatus == VaultInfo::NotInitialized
+                if (oldStatus == VaultInfo::NotInitialized //
                         || newStatus == VaultInfo::NotInitialized) {
                     emit q->isInitializedChanged(newStatus);
                 }
 
-                if (oldStatus == VaultInfo::Creating
-                        || oldStatus == VaultInfo::Opening
-                        || oldStatus == VaultInfo::Closing
+                if (oldStatus == VaultInfo::Creating //
+                        || oldStatus == VaultInfo::Opening //
+                        || oldStatus == VaultInfo::Closing //
                         || oldStatus == VaultInfo::Dismantling) {
                     emit q->isBusyChanged(false);
                 }
@@ -599,12 +586,45 @@ FutureResult<> Vault::forceClose()
 
 FutureResult<> Vault::dismantle(const Payload &payload)
 {
+    const auto resolvedPath = [] (const QString& path) {
+        auto result = QDir(path).canonicalPath();
+        if (!result.endsWith('/')) {
+            result += '/';
+        }
+        return result;
+    };
+
+    const auto resolvedDevice = resolvedPath(d->device.data());
+    const auto resolvedMount  = resolvedPath(d->data->mountPoint.data());
+
+    const auto devices = availableDevices();
+    const int matches = std::count_if(devices.cbegin(), devices.cend(),
+          [&] (const Device& device) {
+              auto thisResolvedDevice = resolvedPath(device.data());
+
+              auto diff = std::mismatch(
+                    resolvedDevice.cbegin(), resolvedDevice.cend(),
+                    thisResolvedDevice.cbegin(), thisResolvedDevice.cend());
+
+              return diff.first == resolvedDevice.cend() ||
+                     diff.second == thisResolvedDevice.cend();
+          });
+
     return
+        matches != 1 ? errorResult(Error::BackendError,
+                                   i18n("Cannot delete the vault; there are other vaults with overlapping paths.")):
+
         // We can not mount something that has not been registered
         // with us before
         !d->data ? errorResult(Error::BackendError,
                                i18n("The vault is unknown; cannot dismantle it.")) :
 
+        // Let's confirm with the user once more
+        QMessageBox::question(nullptr, i18n("Are you sure you want to delete this vault"),
+                                       i18n("This operation will irreversibly delete the following:\n%1\n%2",
+                                            d->device.data(), d->data->mountPoint.data()), QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes ?
+                              errorResult(Error::OperationCancelled, i18n("Delete operation cancelled")) :
+
         // otherwise
         d->followFuture(VaultInfo::Dismantling,
                         d->data->backend->dismantle(d->device, d->data->mountPoint, payload));
